<?php

namespace FiloBlu\OrderAdditionalData\Block\Adminhtml\Order\Tab\View;

use Magento\Backend\Block\Template;
use Magento\Backend\Block\Template\Context;
use Magento\Backend\Block\Widget\Tab\TabInterface;
use Magento\Framework\Api\SimpleDataObjectConverter;
use Magento\Framework\DataObject;
use Magento\Framework\Module\Manager;
use Magento\Framework\Registry;

/**
 * Class AdditionalData
 * @package FiloBlu\OrderAdditionalData\Block\Adminhtml\Ordere\Tab\View
 */
class AdditionalData extends Template implements TabInterface
{
    /**
     * @var string
     */
    protected $_template = 'tab/view/additional_data.phtml';
    /**
     * @var Registry
     */
    protected $_coreRegistry;
    /**
     * @var Manager
     */
    private $moduleManager;

    /**
     * AdditionalData constructor.
     * @param Context $context
     * @param Registry $registry
     * @param Manager $moduleManager
     * @param array $data
     */
    public function __construct(
        Context $context,
        Registry $registry,
        Manager $moduleManager,
        array $data = []
    ) {
        parent::__construct($context, $data);
        $this->_coreRegistry = $registry;
        $this->moduleManager = $moduleManager;
    }

    /**
     * @return mixed
     */
    public function getOrderIncrementId()
    {
        return $this->getOrder()->getIncrementId();
    }

    /**
     * @return mixed|null
     */
    public function getOrder()
    {
        return $this->_coreRegistry->registry('current_order');
    }

    /**
     * Return Tab label
     *
     * @return string
     * @api
     */
    public function getTabLabel()
    {
        return __('Additional Data');
    }

    /**
     * Return Tab title
     *
     * @return string
     * @api
     */
    public function getTabTitle()
    {
        return __('Additional Data');
    }

    /**
     * Can show tab in tabs
     *
     * @return bool
     * @api
     */
    public function canShowTab()
    {
        return true;
    }

    /**
     * Tab is hidden
     *
     * @return bool
     * @api
     */
    public function isHidden()
    {
        return false;
    }

    /**
     * @return mixed
     */
    public function getLastSectionId()
    {
        $sectionsKeys = array_keys($this->getSections());
        return end($sectionsKeys);
    }

    /**
     * @return array
     */
    public function getSections()
    {
        $sections = [];

        $sections['general'] = [
            'pay_exchangerate'                   => [
                'editable' => false,
                'label'    => 'Pay Exchange Rate'
            ],
            'fbnetsuite_theoric_shipping_amount' => [
                'editable' => true,
                'label'    => 'Theoric Shipping Amount'
            ],
        ];

        /**
         * @see https://bitbucket.org/filoblu/mod2-checkout-extend/src/b72ec18fa6bab80c1f4b5d971be5eb407a07bbfc/etc/extension_attributes.xml#lines-8:16
         */
        if ($this->moduleManager->isEnabled('FiloBlu_CheckoutExtend')) {
            $sections['checkout'] = [
                'checkout_invoice'               => [
                    'editable' => true,
                    'label'    => 'Invoice'
                ],
                'checkout_invoice_vat_number'    => [
                    'editable' => true,
                    'label'    => 'VAT Number'
                ],
                'checkout_invoice_fiscal_code'   => [
                    'editable' => true,
                    'label'    => 'Fiscal Code'
                ], 'checkout_invoice_sdi'        => [
                    'editable' => true,
                    'label'    => 'SDI'
                ],
                'checkout_invoice_pec'           => [
                    'editable' => true,
                    'label'    => 'PEC'
                ],
                'checkout_invoice_business_name' => [
                    'editable' => true,
                    'label'    => 'Business Name'
                ],
                'dni'                            => [
                    'editable' => true,
                    'label'    => 'DNI'
                ]
            ];
        }

        if ($this->moduleManager->isEnabled('Ebizmarts_MailChimp')) {
            $sections['mailchimp'] = [
                'mailchimp_campaign_id'        => [
                    'editable' => false,
                    'label'    => 'Campaign Id'
                ],
                'mailchimp_abandonedcart_flag' => [
                    'editable' => false,
                    'label'    => 'Abandonedcart Flag'
                ],
                'mailchimp_landing_page'       => [
                    'editable' => false,
                    'label'    => 'Landing page'
                ],
                'mailchimp_flag'               => [
                    'editable' => false,
                    'label'    => 'Flag'
                ]
            ];
        }

        if ($this->moduleManager->isEnabled('FiloBlu_Storelocator')) {
            $sections['pickup_in_store'] = [
                'pickup_in_store_id'                    => ['editable' => false],
                'pickup_in_store_email'                 => ['editable' => false],
                'pickup_in_store_confirm_date'          => ['editable' => false],
                'pickup_in_store_hash'                  => ['editable' => false],
                'pickup_in_store_confirm_received_date' => ['editable' => false],
            ];
        }

        if ($this->moduleManager->isEnabled('FiloBlu_RefiloPayments')) {
            $root= 'order/extension_attributes/fiscal_information';
            $sections['fiscal_information'] = [
                "{$root}/country_code" =>[
                  'editable' => false,
                  'label' => 'Country'
                ],
                "{$root}/request_invoice"              => [
                    'editable' => false,
                    'label'    => 'Invoice requested'
                ],
                "{$root}/national_identification_number" => [
                    'editable' => false,
                    'label'    => 'National Identification Number'
                ],
                "{$root}/vat_number" => [
                   'editable' => false,
                   'label'    => 'Vat number'
                ],
                "{$root}/destination_code" => [
                    'editable' => false,
                    'label'    => 'Destination Code'
                ],
                "{$root}/company_name" => [
                    'editable' => false,
                    'label'    => 'Company Name'
                ],
                "{$root}/pec_email" => [
                    'editable' => false,
                    'label'    => 'Pec email'
                ],
                "{$root}/public_hash" => [
                    'editable' => false,
                    'label'    => 'Public Hash'
                ],
                "{$root}/document" => [
                    'editable' => false,
                    'label'    => 'Document'
                ],
                "{$root}/dob" => [
                    'editable' => false,
                    'label'    => 'Date of birth'
                ],
                "{$root}/customer_id" => [
                    'editable' => false,
                    'label'    => 'Customer Id'
                ]
            ];
        }

        return $sections;
    }

    /**
     * @param $snakeCaseString
     * @param null $remove
     * @return string
     */
    public function labelize($snakeCaseString, $remove = null): string
    {
        if ($remove !== null) {
            $snakeCaseString = str_replace($remove, '', $snakeCaseString);
        }
        return ucfirst(
            trim(
                SimpleDataObjectConverter::camelCaseToSnakeCase(
                    str_replace('_', ' ', $snakeCaseString)
                )
            )
        );
    }

    /**
     * @return string
     */
    public function getSubmitUrl()
    {
        return $this->getUrl('filoblu_order_additional_data/order/save', ['id' => $this->getOrderId()]);
    }

    /**
     * @return mixed
     */
    public function getOrderId()
    {
        return $this->getOrder()->getEntityId();
    }

    /**
     * TODO: move this function in module core
     *
     * Get object data by path
     *
     * Method consider the path as chain of keys: a/b/c => ['a']['b']['c']
     *
     * @param string $path
     * @return mixed
     */
    public function getDataByPath($path)
    {
        $this->setData('order', $this->_coreRegistry->registry('current_order'));

        $keys = explode('/', $path);

        $data = $this->_data;
        foreach ($keys as $key) {
            if ((array)$data === $data && isset($data[$key])) {
                $data = $data[$key];
            } elseif ($data instanceof DataObject) {
                $data = $data->getDataByKey($key);
            } elseif (method_exists($data, ($method = $this->keyToGetter($key)))) {
                $data  = $data->{$method}();
            } else {
                return null;
            }
        }
        return $data;
    }

    /**
     * @param $key
     * @return string
     */
    public function keyToGetter($key)
    {
        return'get' . str_replace('_', '', ucwords($key, '_'));
    }
}
