<?php
/**
 *
 * Copyright © 2016 Magento. All rights reserved.
 * See COPYING.txt for license details.
 */

namespace FiloBlu\OrderAdditionalData\Model\Plugin;

use Exception;
use FiloBlu\OrderAdditionalData\Model\ItemDataFactory;
use Magento\Framework\Exception\CouldNotSaveException;
use Magento\Framework\Serialize\SerializerInterface;
use Magento\Sales\Api\Data\OrderInterface;
use Magento\Sales\Api\Data\OrderItemInterface;
use Magento\Sales\Api\OrderItemRepositoryInterface;

/*
 * Save Additional Data when saving Order Items by Repository
 */

class ItemSave
{
    /** @var ItemDataFactory */
    protected $itemDataFactory;

    /** @var \Magento\GiftMessage\Api\OrderItemRepositoryInterface */
    protected $orderItemRepository;

    /** @var SerializerInterface */
    protected $_serializer;

    /**
     * ItemSave constructor.
     * @param ItemDataFactory $itemDataFactory
     * @param SerializerInterface $serializer
     */
    public function __construct(
        ItemDataFactory $itemDataFactory,
        SerializerInterface $serializer
    ) {
        $this->itemDataFactory = $itemDataFactory;
        $this->_serializer = $serializer;
    }

    /**
     * @param OrderItemRepositoryInterface $subject
     * @param OrderItemInterface $resultItem
     * @return OrderInterface|OrderItemInterface
     */
    public function afterSave(
        OrderItemRepositoryInterface $subject,
        OrderItemInterface $resultItem
    ) {
        /** @var OrderInterface $resultItem */
        $resultItem = $this->saveItemAdditionalData($resultItem);
        return $resultItem;
    }

    /**
     * @param OrderItemInterface $item
     * @return OrderItemInterface
     * @throws CouldNotSaveException
     */
    protected function saveItemAdditionalData(OrderItemInterface $item)
    {
        $extensionAttributes = $item->getExtensionAttributes();
        if (
            null !== $extensionAttributes &&
            null !== $extensionAttributes->getFbOad()
        ) {
            $additionalDataObject = $extensionAttributes->getFbOad();
            $additionalData = [];
            foreach ($additionalDataObject->getCustomAttributes() as $customAttribute) {
                $additionalData[$customAttribute->getAttributeCode()] = $customAttribute->getValue();
            }
            try {
                $itemData = $this->itemDataFactory->create()->load($item->getItemId(), 'item_id');
                $itemData->setItemId($item->getItemId());
                $itemData->setAdditionalData($this->_serializer->serialize($additionalData));
                $itemData->save();
            } catch (Exception $e) {
                throw new CouldNotSaveException(
                    __('Could not save Additional Data for item: "%1"', $e->getMessage()),
                    $e
                );
            }
        }
        return $item;
    }
}
